#Regression models
from sklearn.ensemble import RandomForestRegressor
from sklearn.model_selection import train_test_split
from sklearn.model_selection import RandomizedSearchCV
from sklearn.metrics import r2_score, mean_squared_error
import xgboost as xgb
from sklearn.neighbors import KNeighborsRegressor
#import logger
from pneuro import appLogger

class RegressionModelTuner():
    """
            This class shall be used to get the best suited Regression model

            Written By: iNeuron
            Version: 1.0
            Revisions: None

    """

    def __init__(self):
      #self.file_object = open('RegressionLogs.txt', 'a+')
      self.file_object = open('pneuro/logs/RegressionLogs.txt', 'a+')
      self.logger_object = appLogger.App_Logger()

    def get_tuned_knn_model(self, x_train, y_train):
        """
            Method Name : get_tuned_knn_model

            Description : This method will be used to get the hypertuned KNN Model

            x_train : Feature Columns of Training DataSet

            y_train : Target Column of Training DataSet

            output : A hyper parameter tuned model object

            Written By: Purvansh Singh

            Version : 1.0

            Revision : None
        """
        try:
            self.logger_object.log(self.file_object,
                                   "KNN Model Training Started.")
            knn_parameters = {'n_neighbors': [50, 100, 200, 250, 300, 350],
                              'weights': ['uniform', 'distance'],
                              'algorithm': ['ball_tree', 'kd_tree'],
                              'leaf_size': [20, 25, 30, 35, 40, 45, 50],
                              }
            rmdsearch = RandomizedSearchCV(KNeighborsRegressor(), knn_parameters, n_iter=10, cv=10, random_state=22,
                                           n_jobs=-1)
            rmdsearch.fit(x_train, y_train)
            hyperparameters = rmdsearch.best_params_
            n_neighbors, weights, algorithm, leaf_size = hyperparameters['n_neighbors'], hyperparameters['weights'], \
                                                         hyperparameters['algorithm'], hyperparameters['leaf_size']
            model = KNeighborsRegressor(n_neighbors=n_neighbors,
                                        weights=weights,
                                        algorithm=algorithm,
                                        leaf_size=leaf_size,
                                        n_jobs=-1)
            model.fit(x_train, y_train)
            self.logger_object.log(self.file_object,
                                   "KNN Model successfully trained.")
            return model
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in performing Model Building and Tuning. Exception message: ' + str(
                                       e))
            raise Exception()

    def get_tuned_random_forest_model(self, x_train, y_train):
        """
        Method Name: get_tuned_random_forest_classifier

        Description: This method will be used to build RandomForestRegressor model

        Input Description:

        x_train : Feature Columns of Training DataSet

        y_train : Target Column of Training DataSet

        On Failure: Raise Exception

        Written By: Purvansh Singh

        Version: 1.0

        Revisions: None

        """
        try:
            self.logger_object.log(self.file_object,
                                   "Random Forest Model Training Started.")
            self.rf_parameters = {
                'max_depth': [5, 10, 15, 20, 25, None],
                'n_estimators': range(10, 500, 50),
                'criterion': ['mse', 'mae'],
                'bootstrap': [True, False],
                'min_samples_split': range(2, 10, 1),
                'max_features': ['auto', 'log2'],
                'min_samples_leaf': range(1, 10),
            }

            self.rmdsearch = RandomizedSearchCV(RandomForestRegressor(), self.rf_parameters, n_iter=10, cv=10,
                                                random_state=22,
                                                n_jobs=-1)
            self.rmdsearch.fit(x_train, y_train)
            hyperparameters = self.rmdsearch.best_params_
            max_depth, n_estimators, criterion, bootstrap, min_samples_split, max_features, min_samples_leaf = \
                hyperparameters['max_depth'], hyperparameters['n_estimators'], hyperparameters['criterion'], \
                hyperparameters['bootstrap'], hyperparameters['min_samples_split'], hyperparameters['max_features'], \
                hyperparameters['min_samples_leaf']

            self.model = RandomForestRegressor(n_estimators=n_estimators,
                                               max_depth=max_depth,
                                               criterion=criterion,
                                               min_samples_leaf=min_samples_leaf,
                                               max_features=max_features,
                                               min_samples_split=min_samples_split,
                                               bootstrap=bootstrap,
                                               random_state=25,
                                               n_jobs=-1)

            self.model = RandomForestRegressor(n_jobs=-1)
            self.model.fit(x_train, y_train)
            self.logger_object.log(self.file_object,
                                   "Random Forest Model successfully Trained.")
            return self.model
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in performing Model Building and Tuning. Exception message: ' + str(
                                       e))
            raise Exception()

    def get_tuned_xgboost_model(self, x_train, y_train):
        """
        Method Name: get_tuned_xgboost_model

        Description: This method will be used to build XGBoost Regressor model

        Input Description:

        x_train : Feature Columns of Training DataSet

        y_train : Target Column of Training DataSet

        On Failure: Raise Exception

        Written By: Purvansh Singh

        Version: 1.0

        Revisions: None

        """
        try:
            self.logger_object.log(self.file_object,
                                   "Xgboost Model Training Started.")
            self.xg_parameters = {"n_estimators": [10, 50, 100, 200],
                                  "learning_rate": [0.05, 0.10, 0.15, 0.20, 0.25, 0.30],
                                  "max_depth": [3, 4, 5, 6, 8, 10, 12, 15, 20],
                                  "min_child_weight": [1, 3, 5, 7],
                                  "gamma": [0.0, 0.1, 0.2, 0.3, 0.4, 0.5],
                                  "colsample_bytree": [0.3, 0.4, 0.5, 0.7]
                                  }

            self.rmdsearch = RandomizedSearchCV(xgb.XGBRegressor(objective='reg:squarederror'),param_distributions=self.xg_parameters, n_iter=10, cv=10, n_jobs=-1)
            self.rmdsearch.fit(x_train, y_train)
            hyperparameters = self.rmdsearch.best_params_
            n_estimators, min_child_weight, max_depth, learning_rate, gamma, colsample_bytree = hyperparameters[
                                                                                                    'n_estimators'], \
                                                                                                hyperparameters[
                                                                                                    'min_child_weight'], \
                                                                                                hyperparameters[
                                                                                                    'max_depth'], \
                                                                                                hyperparameters[
                                                                                                    'learning_rate'], \
                                                                                                hyperparameters[
                                                                                                    'gamma'], \
                                                                                                hyperparameters[
                                                                                                    'colsample_bytree']
            self.xgboost_model = xgb.XGBRegressor(n_estimators=n_estimators,
                                               learning_rate=learning_rate,
                                               gamma=gamma,
                                               min_child_weight=min_child_weight,
                                               max_depth=max_depth,
                                               colsample_bytree=colsample_bytree)

            self.xgboost_model = xgb.XGBRegressor(objective='reg:squarederror',n_jobs=-1)
            self.xgboost_model.fit(x_train, y_train)
            self.logger_object.log(self.file_object,
                                   "Xgboost Model Training Started.")
            return self.xgboost_model
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in performing Model Building and Tuning. Exception message: ' + str(
                                       e))
            raise Exception()


import os

#from file_methods import FileOperation
#from appLogger import AppLogger
from pneuro import appLogger
from pneuro import file_methods
from pneuro import preProcess
#from preProcess import Preprocessor
class CustomException(Exception):
    pass

class Prediction():
    """
        This class shall  be used for prediction.

        Written By: iNeuron Intelligence
        Version: 1.0
        Revisions: None

        """

    def __init__(self):
        """
                        Method Name: __init__
                        Description: This method is for attributes initialization
                        Output: attributes

                        Written By: iNeuron Intelligence
                        Version: 1.0
                        Revisions: None

                """
        self.file_object = open('pneuro/logs/predictionLogs.txt', 'a+')
        self.logger = appLogger.AppLogger()
        self.file_operation = file_methods.FileOperation()
        self.models_directory = 'models'
        self.preprocessor = preProcess.Preprocessor()

    def predict_results(self, data, target_column, orig_data_col):
        """
            Method Name: predict_results
            Description: This method is for predicting the results of the test data
            Output: attributes
            Written By: iNeuron Intelligence
            Version: 1.0
            Revisions: None
      """
        model_name = os.chdir(self.models_directory)
        model_file_name = os.listdir()[0]
        os.chdir('../')
        #loading the best model from the directory
        model = self.file_operation.load_model(model_file_name)
        #adding the dummy target column for preprocessing the predict data
        data[target_column]=0
        difList = [string for string in orig_data_col if string not in data.columns]
        if len(difList)>0:
            self.logger.log(self.file_object,
                                   'The predict data structure is differing from trained data structure')
            raise RuntimeError("The predict data structure is differing from trained data structure. Please provide valid data ")
        try:
            #preprocess the data
            x, y = self.preprocessor.preprocess(data, target_column)
            self.logger.log(self.file_object,
                                   'Preprocessing the data to be predicted')
        except Exception as e:
            self.logger.log(self.file_object,
                                   'E : Exception occured during Preprocessing of Predictable Data' + str(
                                       e))
            raise Exception()
        self.logger.log(self.file_object,
                               'After preprocessing the data to be predicted')
        try:
            results = model.predict(x)
            data['prediction'] = results
            data.drop(target_column, axis=1, inplace=True)
        except Exception as e:
            self.logger.log(self.file_object,
                                   'E : Exception occured during predicting the data after preprocessing' + str(
                                       e))
            raise Exception()
        self.logger.log(self.file_object,
                               'Model Prediction completed')
        return data



import pandas as pd
import numpy as np
from sklearn.decomposition import PCA
from imblearn.over_sampling import RandomOverSampler, SMOTE
from imblearn.under_sampling import RandomUnderSampler
from sklearn.feature_selection import VarianceThreshold
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import MinMaxScaler

#from appLogger import AppLogger
from pneuro import appLogger

class Preprocessor():
    """
        This class shall be used to include all Data Pre-processing techniques to be
        fed to the Machine Learning Models

        Written By: iNeuron Intelligence
        Version: 1.0
        Revisions: None

    """

    def __init__(self):
        self.file_object = open('pneuro/logs/dataPreprocessingLogs.txt', 'a+')
        self.logger_object = appLogger.AppLogger()

    def get_data_profile(self, data):
        """
                        Method Name: get_data_profile
                        Description: This method will be used to do data-profiling

                        Input Description:
                        data: Name of the input dataframe

                        On Exception: Write the exception in the log file. Raise an exception with the appropriate error message

                        Output: The full data profile report using a dictionary

                        Written By: iNeuron
                        Version: 1.0
                        Revisions: None
                        """
        self.data_profile = {}
        self.missing_values = {}
        self.missing_val_pct = {}
        self.data_profile['rows'] = data.shape[0]
        self.data_profile['columns'] = data.shape[1]
        self.missing_vals = data.isna().sum()
        for col in data.columns:
            if data[col].isnull().sum() > 0:
                self.missing_values[col] = data[col].isnull().sum()
                self.missing_val_pct[col] = (data[col].isnull().sum() / len(data)) * 100
        self.data_profile['missing_values'] = self.missing_values
        self.data_profile['missing_vals_pct'] = self.missing_val_pct
        self.data_profile['categorical_columns'] = list(data.select_dtypes(exclude=["int64", "float"]))
        self.data_profile['num_categorical_columns'] = len(self.data_profile['categorical_columns'])
        self.data_profile['numerical_columns'] = list(data.select_dtypes(exclude=["object"]))
        self.data_profile['num_numerical_columns'] = len(self.data_profile['numerical_columns'])
        self.data_profile['num_duplicate_rows'] = data.duplicated().sum()
        self.describe = data.describe().T
        self.standard_deviation = self.describe[self.describe['std'] == 0]
        self.standard_deviation = list(self.standard_deviation.index)
        self.data_profile['num_col_with_std_zero'] = len(self.standard_deviation)
        if len(self.standard_deviation) > 0:
            self.data_profile['cols_with_std_zero'] = self.standard_deviation

        self.size = data.size / (1024 * 1024)
        self.data_profile['datasize'] = str(round(self.size, 2)) + " MB"

        return self.data_profile

    def separate_label_feature(self, data, label_column_name):
        """
                        Method Name: separate_label_feature
                        Description: This method separates the features and a Label Coulmns.
                        Output: Returns two separate Dataframes, one containing features and the other containing Labels .
                        On Failure: Raise Exception

                        Written By: iNeuron Intelligence
                        Version: 1.0
                        Revisions: None

                """
        self.logger_object.log(self.file_object, 'Entered the separate_label_feature method of the Preprocessor class')

        try:
            self.X = data.drop(labels=label_column_name,axis=1)  # drop the columns specified and separate the feature columns
            self.Y = data[label_column_name]  # Filter the Label columns
            self.logger_object.log(self.file_object,
                                   'Label Separation Successful. Exited the separate_label_feature method of the Preprocessor class')
            return self.X, self.Y
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in separate_label_feature method of the Preprocessor class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'Label Separation Unsuccessful. Exited the separate_label_feature method of the Preprocessor class')
            raise Exception()

    def remove_columns(self, data, columns):
        """
                Method Name: remove_columns
                Description: This method removes the given columns from a pandas dataframe.
                Output: A pandas DataFrame after removing the specified columns.
                On Failure: Raise Exception

                Written By: iNeuron Intelligence
                Version: 1.0
                Revisions: None

        """
        self.logger_object.log(self.file_object, 'Entered the remove_columns method of the Preprocessor class')
        self.data = data
        self.columns = columns
        try:
            self.useful_data = self.data.drop(labels=self.columns, axis=1)  # drop the labels specified in the columns
            self.logger_object.log(self.file_object,
                                   'Column removal Successful.Exited the remove_columns method of the Preprocessor class')
            return self.useful_data
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in remove_columns method of the Preprocessor class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'Column removal Unsuccessful. Exited the remove_columns method of the Preprocessor class')
            raise Exception()

    def impute_missing_values(self, data, mv_flag=None, target=None, strategy='mode', impute_val=None,
                              missing_vals=None):
        """
                Method Name: impute_missing_values
                Description: This method will be used to impute missing values in the dataframe

                Input Description:
                data: Name of the input dataframe

                target: Name of the target column of DataFrame

                strategy: Strategy to be used for MVI (Missing Value Imputation)
                --‘median’ : default for continuous variables,
                 replaces missing value(s) with median of the concerned column
                --‘mean’
                --‘mode’ : default for categorical variables
                --‘fixed’ : replaces all missing values with a fixed ‘explicitly specified’ value

                impute_val: None(default), can be assigned a value to be used for imputation i
                n ‘fixed’ strategy

                missing_vals: None(default), a list/tuple of missing value indicators. By default,
                 it considers only NaN as missing. Dictionary can be passed to consider different missing values
                for different columns in format – {col_name:[val1,val2, …], col2:[…]}

                mv_flag: None(default), can be passed list/tuple of columns as input for which it creates missing
                value flags

                On Exception: Write the exception in the log file. Raise an exception with the appropriate error message

                return: A DataFrame with missing values imputed

                Written By: Purvansh singh
                Version: 1.0
                Revisions: None
                """
        self.logger_object.log(self.file_object, "Entered into impute_missing_values method.")
        try:
            if isinstance(data, pd.DataFrame) and not data.empty:
                self.logger_object.log(self.file_object, "Non-empty DataFrame object Identified")
                dataframe = data

                if mv_flag is True:
                    self.logger_object.log(self.file_object, "my_flag found True Imputing Dataframe.")
                    # Converting missing_vals to Nan Values
                    if missing_vals:
                        dataframe.replace(missing_vals, np.nan, inplace=True)
                    #  Checking for Missing Values in Dependent Variable
                    if dataframe[target].isna().any():
                        dataframe = dataframe[dataframe[
                            target].notna()].copy()  # Selecting the Dataframe With No missing values in Dependent column
                    # Checking for Missing Values in Independent Variables
                    Missing_data_columns = dataframe.columns[
                        dataframe.isna().any()].tolist()  # Finding Columns with the missing data from dataframe
                    if strategy == 'fixed':  # checking if strategy == fixed
                        dataframe.fillna(impute_val,
                                         inplace=True)  # Filling the Nan values with the imputed value from user
                    else:
                        for columns in Missing_data_columns:  # Iterating over the columns having Nan Values
                            if dataframe[columns].dtype == 'object':  # Checking for the categorical data
                                mode = dataframe[columns].mode()[0]
                                dataframe[columns].fillna(mode,
                                                          inplace=True)  # Imputing Nan values with mode of the column
                            else:
                                if strategy == 'median':  # checking if the strategy == median
                                    median = dataframe[columns].median()
                                    dataframe[columns].fillna(median,
                                                              inplace=True)  # Imputing Nan values with median of the column
                                else:  # The only strategy remains is mean
                                    mean = dataframe[columns].mean()
                                    dataframe[columns].fillna(mean,
                                                              inplace=True)  # Imputing Nan values with mean of the column

                else:
                    self.logger_object.log(self.file_object, "my_flag found False")
            else:
                raise Exception("No DataFrame Found")
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   "Error Occurred in impute_missing_values, Error statement: " + str(e))
            raise Exception(e) from None  # Suppressing the Error Chaining
        else:
            self.logger_object.log(self.file_object, "Imputed DataFrame Returned Successfully")
            return dataframe

    def type_conversion(self, dataset, cat_to_num=None, num_to_cat=None):
        '''

            Method Name: type_conversion
            Description: This method will be used to convert column datatype from
            numerical to categorical or vice-versa, if possible.

            Input Description:

            dataset: input DataFrame in which type conversion is needed

            cat_to_num: None(default),list/tuple of variables that need to
            be converted from categorical to numerical

            num_to_cat: None(default),list/tuple of variables to be
            converted from numerical to categorical

            return: A DataFrame with column types changed as per requirement

            On Exception : Write the exception in the log file. Raise an exception with the appropriate error message

            Written By: Purvansh singh
            Version: 1.0
            Revisions: None
        '''
        self.logger_object.log(self.file_object, "Entered into type_conversion method.")
        try:
            if isinstance(dataset, pd.DataFrame) and not dataset.empty:
                self.logger_object.log(self.file_object, "Non-empty DataFrame object Identified")
                if cat_to_num is not None:
                    for column in cat_to_num:
                        dataset[column] = pd.to_numeric(dataset[column])

                if num_to_cat is not None:
                    for column in num_to_cat:
                        dataset[column] = dataset[column].astype('object')
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   "Error Occurred in type_conversion method, Error statement: " + str(e))
            raise Exception(e) from None  # Suppressing the Error Chaining
        else:
            self.logger_object.log(self.file_object, "type_converted DataFrame Returned Successfully")
            return dataset

    def remove_imbalance(self, data, target, threshold=10.0, oversample=True, smote=False):
        """
        Method Name: remove_imbalance
        Description: This method will be used to handle unbalanced datasets(rare classes) through oversampling/ undersampling
                     techniques
        Input Description: data: the input dataframe with target column.
                           threshold: the threshold of mismatch between the target values to perform balancing.

        Output: A balanced dataframe.
        On Failure: Raise Exception

        Written By: Punit Nanda
        Version: 1.0
        Revisions: None

        """
        self.logger_object.log(self.file_object,
                               'Entered the remove_imbalance method of the DataPreprocessor class')  # Logging entry to the method
        try:
            # data= pd.read_csv(self.training_file) # reading the data file
            self.logger_object.log(self.file_object,
                                   'DataFrame Load Successful of the remove_imbalance method of the DataPreprocessor class')
            # return self.data # return the read data to the calling method

            self.logger_object.log(self.file_object,
                                   'X y created in the remove_imbalance method of the DataPreprocessor class')
            X = data.drop(target, axis=1)
            y = data[target]

            self.logger_object.log(self.file_object,
                                   'Class Imbalance Process Starts in the remove_imbalance method of the DataPreprocessor class')

            no_of_classes = data[target].nunique()

            if no_of_classes == 2:

                self.logger_object.log(self.file_object,
                                       'No of Classes is 2 in the remove_imbalance method of the DataPreprocessor class')
                thresh_satisfied = ((data[target].value_counts() / float(len(data[target])) * 100).any() < threshold)
                if thresh_satisfied:
                    self.logger_object.log(self.file_object,
                                           'Threshold satisfied in the remove_imbalance method of the DataPreprocessor class')
                    if smote:
                        self.logger_object.log(self.file_object,
                                               'OverSampling using SMOTE having 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        smote = SMOTE()
                        X, y = smote.fit_resample(X, y)
                    elif oversample:
                        self.logger_object.log(self.file_object,
                                               'OverSampling minority classes data having 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        ROS = RandomOverSampler(sampling_strategy='auto', random_state=42)
                        X, y = ROS.fit_sample(X, y)
                    else:
                        self.logger_object.log(self.file_object,
                                               'UnderSampling majority classes data having 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        ROS = RandomUnderSampler(sampling_strategy='auto', random_state=42)
                        X, y = ROS.fit_sample(X, y)
            else:

                high = (data[target].value_counts() / float(len(data[target])) * 100).ravel().max()
                low = (data[target].value_counts() / float(len(data[target])) * 100).ravel().min()

                thresh_satisfied = (high - low > 100.0 - threshold)

                if thresh_satisfied:
                    self.logger_object.log(self.file_object,
                                           'Threshold satisfied in the remove_imbalance method of the DataPreprocessor class')
                    if smote:
                        self.logger_object.log(self.file_object,
                                               'OverSampling using SMOTE having more than 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        for i in range(no_of_classes - 2):
                            smote = SMOTE()
                            X, y = smote.fit_resample(X, y)
                    elif oversample:
                        self.logger_object.log(self.file_object,
                                               'OverSampling minority classes data having more than 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        for i in range(no_of_classes - 2):
                            ROS = RandomOverSampler(sampling_strategy='auto', random_state=42)
                            X, y = ROS.fit_sample(X, y)
                    else:
                        self.logger_object.log(self.file_object,
                                               'UnderSampling majority classes data having more than 2 classes in the remove_imbalance method of the DataPreprocessor class')
                        for i in range(no_of_classes - 2):
                            ROS = RandomUnderSampler(sampling_strategy='auto', random_state=42)
                            X, y = ROS.fit_sample(X, y)

            y.to_frame(name=target)
            dfBalanced = pd.concat([X, y], axis=1)
            self.logger_object.log(self.file_object,
                                   'Class Imbalance Process Ends in the remove_imbalance method of the DataPreprocessor class')
            return dfBalanced

        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in remove_imbalance method of the DataPreprocessor class. Exception message: ' + str(
                                       e))  # Logging the exception message
            self.logger_object.log(self.file_object,
                                   'DataFrame Load Unsuccessful.Exited the remove_imbalance method of the DataPreprocessor class')  # Logging unsuccessful load of data
            raise Exception()  # raising exception and exiting

    def remove_columns_with_minimal_variance(self, data, threshold):
        """
        Method Name: remove_columns_with_minimal_variance
        Description: This method drops any numerical column with standard deviation below specified threshold
        Input Parameter Description: data: input DataFrame in which we need to check std deviations
                                     threshold : the threshold for std deviation below which we need to drop the columns


        Output: A DataFrame with numerical columns with low std dev dropped.
        On Failure: Raise Exception

        Written By: PUNIT NANDA
        Version: 1.0
        Revisions: None

        """
        self.logger_object.log(self.file_object,
                               'Entered the remove_columns_with_minimal_variance method of the DataPreprocessor class')  # Logging entry to the method
        try:
            sel = VarianceThreshold(threshold=(threshold * (1 - threshold)))
            columnlist = list(data.select_dtypes(include='number').columns)
            sel_var = sel.fit_transform(data[columnlist])
            new_data = data[data.columns[sel.get_support(indices=True)]]
            return new_data  # return the read data to the calling method
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured in remove_columns_with_minimal_variance method of the DataPreprocessor class. Exception message: ' + str(
                                       e))  # Logging the exception message
            raise Exception()  # raising exception and exiting

    def standardize_data(self, dataframe):

        """
        Method Name: standardize_data
        Description: This method will be used to standardize al the numeric variables. Where mean = 0, std dev = 1.
        Input Description: data: the input dataframe with numeric columns.

        Output: Standardized data where mean of each column will be 0 and standard deviation will be 1.
        On Failure: Raise Exception

        Written By: Abhishek Kulkarni
        Version: 1.0
        Revisions: None

        """
        try:
            data = dataframe
            stdscalar = StandardScaler()
            scaled_data = stdscalar.fit_transform(data)
            scaled_data = pd.Dataframe(data=scaled_data, columns=data.columns)
            return scaled_data
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured while stadardizing data. Exception message:  ' + str(e))
            raise Exception()

    def normalize_data(self, dataframe):

        """
        Method Name: normalize_data
        Description: This method will be used to normalize all the numeric variables. Where min value = 0 and max value = 1.
        Input Description: data: the input dataframe with numeric columns.

        Output: Normalized data where minimum value of each column will be 0 and maximum value of each column will be 1.
        On Failure: Raise Exception

        Written By: Abhishek Kulkarni
        Version: 1.0
        Revisions: None

        """
        try:
            data = dataframe
            normalizer = MinMaxScaler()
            normalized_data = normalizer.fit_transform(data)
            normalized_data = pd.Dataframe(data=normalized_data, columns=data.columns)
            return normalized_data
        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'Exception occured while stadardizing data. Exception message:  ' + str(e))
            raise Exception()

    def pca(self, data, var_explained):

        """
                Method Name: pca
                Description: This method reduces the dimension from scaled Data which enables
                             quick for large data files.

                input      : Data which is Scaled, var_explained = 0.90(default value)

                Output     : It returns the scaled and reduced dimensions.

                On Failure : Raise Exception

                Written by : Saravanan Dhanapal

                version    : 1.0

                revisions  : None.


                """

        self.data = data
        self.var_explained = var_explained
        self.logger_object.log(self.file_object, 'S::Entered the PCA method of the DataPreprocessor class')
        try:
            n = len(data.keys())  # find out the no columns in the data
            mat_pca = PCA(n_components=n)
            mat_pca.fit(data)  # applying PCA model

            ##calculate variance ratios
            variance = mat_pca.explained_variance_ratio_
            cum_var = np.cumsum(np.round(mat_pca.explained_variance_ratio_, decimals=3) * 100)
            self.logger_object.log(self.file_object,
                                   'I : PCA params variance and cum_var are :' + str(variance) + str(cum_var))

            # function for calculating number of principal components to use:
            def calc_num_components(cum_var, var_explained):
                for i in range(n):
                    if cum_var[i] >= var_explained:
                        return i + 1

            # call the function to calulate num_components:
            n_components = calc_num_components(cum_var, var_explained)
            self.logger_object.log(self.file_object, 'I : PCA n_components:' + str(n_components))

            # create the PCA instance
            pca = PCA(n_components=n_components)
            principal_components = pca.fit_transform(data)

            # Convert into dataframe
            pca_data = pd.DataFrame(data=principal_components,
                                    columns=['PC' + str(i) for i in range(1, n_components + 1)])
            self.logger_object.log(self.file_object, 'C:: Compeleted the PCA method of the DataPreprocessor class')
            return pca_data

        except Exception as e:
            self.logger_object.log(self.file_object,
                                   'E : Exception occured in PCA method of the Data Preprocessor class. Exception message:  ' + str(
                                       e))
            self.logger_object.log(self.file_object,
                                   'E : Unsuccessful. Exited the PCA method of the DataPreprocessor class')

            raise Exception()

    def preprocess(self, dataset, target_column,unwanted_cols):
        dataset = self.remove_columns(dataset, columns=['Unnamed: 0'])
        dataset = self.impute_missing_values(data=dataset, mv_flag=True, target=target_column)
        # dataset = self.remove_columns(dataset, unwanted_cols)
        dataset = self.remove_imbalance(dataset, target_column, threshold=10.0, oversample=True, smote=False)
        dataset, self.y = self.separate_label_feature(dataset, target_column)
        self.x = self.remove_columns_with_minimal_variance(data=dataset, threshold=0.1)




        return self.x, self.y



"""
from dataLoad.data_loader import DataGetter
from .dataPreprocessing.preProcess import Preprocessor
from .dataVisualization.visualization import DataVisualization
from .modelTraining.classificationModels import ClassificationModelTuner
from .prediction.predict import Prediction
from .logger.appLogger import AppLogger
"""


from datetime import datetime


class AppLogger:
    def __init__(self):
        self.now = datetime.now()
        self.date = self.now.date()
        self.current_time = self.now.strftime("%H:%M:%S")

    def log(self, file_object, log_message):
        """This method will be used for logger all the information to the file."""

        file_object.write(
            str(self.date) + "/" + str(self.current_time) + "\t\t" + log_message + "\n")



import pandas as pd
# import pymongo
# import os
#from appLogger import AppLogger
from pneuro import appLogger


class DataGetter():
    """
    This class shall  be used for obtaining the data from the source for training.

    Written By: iNeuron Intelligence
    Version: 1.0
    Revisions: None

    """

    def __init__(self):
        """
                Method Name: __init__
                Description: This method is for attributes initialization
                Output: attributes

                Written By: iNeuron Intelligence
                Version: 1.0
                Revisions: None

        """
        """
        self.log_file = 'logs/dataLoadLogs.txt'
        if os.path.exists(self.log_file):
            self.file_mode = 'a+'  # append if already exists
        else:
            self.file_mode = 'w'  # make a new file if not
        print(self.file_mode)
        print(os.getcwd())
        self.log_file_name = open(self.log_file,self.file_mode)
        """
        self.log_file_name = open('dataLoadLogs.txt', 'a+')
        self.logger = appLogger.AppLogger()

    def read_data_from_csv(self, file_name):
        """
                Method Name: read_data_from_csv
                Description: This method reads the data from a CSV file.
                Output: A pandas DataFrame.
                On Failure: Raise Exception

                Written By: iNeuron Intelligence
                Version: 1.0
                Revisions: None

                """
        self.logger.log(self.log_file_name, 'Entered the read_data_from_csv method of the Data_Getter class')
        try:
            self.csv = pd.read_csv(file_name, sep=',', header='infer', names=None, usecols=None)
            self.logger.log(self.log_file_name,
                            'Data Load Successful.Exited the read_data_from_csv method of the Data_Getter class')
            return self.csv
        except Exception as e:
            self.logger.log(self.log_file_name,
                            'Exception occured in read_data_from_csv method of the Data_Getter class. Exception message: ' + str(
                                e))
            self.logger.log(self.log_file_name,
                            'Data Load Unsuccessful.Exited the read_data_from_csv method of the Data_Getter class')
            raise Exception()
    def read_data_from_excel(self, file_name):
        """
                       Method Name: read_data_from_excel
                       Description: This method reads the data from a excel file.
                       Output: A pandas DataFrame.
                       On Failure: Raise Exception

                       Written By: iNeuron Intelligence
                       Version: 1.0
                       Revisions: None

                       """


        self.logger.log(self.log_file_name, 'Entered the read_data_from_excel method of the Data_Getter class')
        try :
            excel = pd.read_excel(file_name, sheet_name=0, header=0, names=None, index_col=None, usecols=None, )
            self.logger.log(self.log_file_name,'Data Load Successful.Exited the read_data_from_excel method of the Data_Getter class')
            return pd.DataFrame(excel)
        except Exception as e:
            self.logger.log(self.log_file_name,
                            'Exception occured in read_data_from_excel method of the Data_Getter class. Exception message: ' + str(
                                e))
            self.logger.log(self.log_file_name,
                            'Data Load Unsuccessful.Exited the read_data_from_excel method of the Data_Getter class')
            raise Exception()
    def read_data_from_json(self, file_name):
        """
                       Method Name: read_data_from_json
                       Description: This method reads the data from a json file.
                       Output: A pandas DataFrame.
                       On Failure: Raise Exception

                       Written By: iNeuron Intelligence
                       Version: 1.0
                       Revisions: None

                       """


        self.logger.log(self.log_file_name, 'Entered the read_data_from_json method of the Data_Getter class')
        try:
            json = pd.read_json(self, file_name)
            self.logger.log(self.log_file_name, 'Data Load Successful.Exited the read_data_from_json method of the Data_Getter class' )
            return pd.DataFrame(json)
        except Exception as e:
            self.logger.log(self.log_file_name,
                            'Exception occured in read_data_from_json method of the Data_Getter class. Exception message: ' + str(
                                e))
            self.logger.log(self.log_file_name,
                            'Data Load Unsuccessful.Exited the read_data_from_json method of the Data_Getter class')
            raise Exception()


    def read_data_from_html(self, file_name):
        """
                       Method Name: read_data_from_html
                       Description: This method reads the data from a html file.
                       Output: A pandas DataFrame.
                       On Failure: Raise Exception

                       Written By: iNeuron Intelligence
                       Version: 1.0
                       Revisions: None

                       """

        self.logger.log(self.log_file_name, 'Entered the read_data_from_html method of the Data_Getter class')
        try :
            html = pd.read_html(self, file_name)
            data = html[0]
            self.logger.log(self.log_file_name, 'Data Load Successful.Exited the read_data_from_html method of the Data_Getter class')
            return pd.DataFrame(data)
        except Exception as e:
            self.logger.log(self.log_file_name,
                            'Exception occured in read_data_from_html method of the Data_Getter class. Exception message: ' + str(
                                e))
            self.logger.log(self.log_file_name,
                            'Data Load Unsuccessful.Exited the read_data_from_html method of the Data_Getter class')
            raise Exception()
    """
    def Connect_to_sqldb(self):
        self.logger.log(self.log_file_name,'Connecting to SQL DB.')
        connection = sqlalchemy.create_engine("mysql+pymysql://root:swati9601841925@localhost:3306/imdb1")
        self.logger.log(self.log_file_name,'Successfully connected to the SQL DB.')
        return connection
    def read_data_from_sqldb(self,table_name):
        self.logger.log(self.log_file_name,'Reading the data from SQL DB.')
        con = Connect_to_sqldb()
        sql_data = pd.read_sql_table(table_name,con)
        self.logger.log(self.log_file_name,'Successfully read the data.')
        return pd.DataFrame(sql_data)
    def read_data_from_mongdb(self,file_name):
        self.logger.log(self.log_file_name,'Reading the data from mongo DB.')
        clinet= pymongo.MongoClinet("mongodb://127.0.0.1:27017/")
        mydb = client["file_name"]
        self.logger.log(self.log_file_name,'Successfully read the data.')
        return pd.DataFrame(mydb)
    """

    def get_data(self, file_type, file):
        """
                        Method Name: get_data
                        Description: This method reads the data based on the file
                                    format
                        Output: A pandas DataFrame.
                        On Failure: Raise Exception

                        Written By: iNeuron Intelligence
                        Version: 1.0
                        Revisions: None

                        """
        self.logger.log(self.log_file_name, 'Entered the get_data method of '
                                            'the Data_Getter class')
        try:
            if file_type == 'CSV':
                data = self.read_data_from_csv(file)
                self.logger.log(self.log_file_name,
                                    'Data Load Successful.Exited the get_data method of the Data_Getter class')
            if file_type == 'xlsx':
                data = self.read_data_from_excel(file)
                self.logger.log(self.log_file_name,
                                    'Data Load Successful.Exited the get_data method of the Data_Getter class')
            if file_type == 'html':
                data = self.read_data_from_html(file)
                self.logger.log(self.log_file_name,
                                    'Data Load Successful.Exited the get_data method of the Data_Getter class')
            if file_type == 'json':
                data = self.read_data_from_json(file)
                self.logger.log(self.log_file_name,
                                    'Data Load Successful.Exited the get_data method of the Data_Getter class')
            return data
        except Exception as e:
            self.logger.log(self.log_file_name,
                            'Exception occured in get_data method of the Data_Getter class. Exception message: ' + str(
                                e))
            self.logger.log(self.log_file_name,
                            'Data Load Unsuccessful.Exited the get_data method of the Data_Getter class')
            raise Exception()







import pathlib
from setuptools import setup, find_packages

# This call to setup() does all the work

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


PACKAGE_NAME = 'pneuro'

# Read-in the README.md
#with open('README.md', 'r') as f:
#    readme = f.readlines()
#readme = ''.join(readme)

with open('requirements.txt') as f: 
    requirements = f.readlines()


setup(
    name=PACKAGE_NAME,
    long_description="pneuro Library",
    description='Template python package',
    long_description_content_type="text/markdown",
    version="1.2.4",
    author="Punit Nanda",
    author_email="punit.nanda01@gmail.com",
    license="MIT",
    #packages=[  PACKAGE_NAME,
    #            '{}.dataLoad'.format(PACKAGE_NAME),
    #            '{}.dataPreprocessing'.format(PACKAGE_NAME),
    #            '{}.dataVisualization'.format(PACKAGE_NAME),
    #            '{}.fileOperations'.format(PACKAGE_NAME),
    #            '{}.logger'.format(PACKAGE_NAME),
    #            '{}.logs'.format(PACKAGE_NAME),
    #            '{}.models'.format(PACKAGE_NAME),
    #            '{}.modelTraining'.format(PACKAGE_NAME),
    #            '{}.prediction'.format(PACKAGE_NAME),
    #            '{}.static'.format(PACKAGE_NAME),
    #            '{}.templates'.format(PACKAGE_NAME)],
    packages=find_packages(),
    include_package_data=True,                
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    #py_modules=['pneuro.dataLoad.data_loader',
    #            'pneuro.dataPreprocessing.preProcess',
    #            'pneuro.dataVisualization.visualization',
    #            'pneuro.fileOperations.file_methods',
    #            'pneuro.logger.appLogger',
    #            'pneuro.modelTraining.classificationModels',
    #            'pneuro.modelTraining.regressionModels',
    #            'pneuro.prediction.predict'],
    entry_points={'console_scripts': ['pneuro = pneuro.__main__:main']},
    #entry_points ={'console_scripts': ['gfg = vibhu4gfg.gfg:main']}, 
    #package_dir={'':'Autoneuro'},
    install_requires=requirements
)